@@ -235,7 +235,7 @@ bool is_absolute_path(std::string const& f)
 #if defined(TORRENT_WINDOWS) || defined(TORRENT_OS2)
 	int i = 0;
 	// match the xx:\ or xx:/ form
-	while (f[i] && strchr("abcdefghijklmnopqrstuvxyz", f[i])) ++i;
+	while (f[i] && strchr("abcdefghijklmnopqrstuvxyzABCDEFGHIJKLMNOPQRSTUVXYZ", f[i])) ++i;
 	if (i < int(f.size()-1) && f[i] == ':' && (f[i+1] == '\\' || f[i+1] == '/'))
 		return true;
 
@@ -1158,7 +1158,8 @@ BITTORRENT OPTIONS
 
 NETWORK OPTIONS
   -x <file>             loads an emule IP-filter file
-  -Y                    Rate limit local peers)"
+  -Y                    Rate limit local peers
+)"
 #if TORRENT_USE_I2P
 R"(  -i <i2p-host>         the hostname to an I2P SAM bridge to use
 )"
