@@ -35,24 +35,35 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/socket.hpp"
 #include "libtorrent/error_code.hpp"
-#include "libtorrent/broadcast_socket.hpp"
-#include "libtorrent/thread.hpp"
 #include "libtorrent/deadline_timer.hpp"
 #include "libtorrent/enum_net.hpp"
 #include "libtorrent/resolver.hpp"
+#include "libtorrent/debug.hpp"
+#include "libtorrent/string_util.hpp"
+#include "libtorrent/aux_/portmap.hpp"
+#include "libtorrent/aux_/vector.hpp"
+#include "libtorrent/aux_/session_settings.hpp"
+#include "libtorrent/aux_/openssl.hpp" // for ssl::context
 
-#include <boost/function/function1.hpp>
-#include <boost/function/function5.hpp>
-#include <boost/noncopyable.hpp>
-#include <boost/shared_ptr.hpp>
-#include <boost/enable_shared_from_this.hpp>
+#include <memory>
+#include <functional>
 #include <set>
 
-namespace libtorrent
-{
+namespace libtorrent {
 	struct http_connection;
 	class http_parser;
 
+namespace aux {
+
+	struct socket_package
+	{
+		socket_package(io_service& ios) : socket(ios) {}
+		udp::socket socket;
+		std::array<char, 1500> buffer;
+		udp::endpoint remote;
+	};
+}
+
 	namespace upnp_errors
 	{
 		// error codes for the upnp_error_category. They hold error codes
@@ -70,7 +81,7 @@ namespace libtorrent
 			// The source IP address cannot be wild-carded, but
 			// must be fully specified
 			source_ip_cannot_be_wildcarded = 715,
-			// The external port cannot be wildcarded, but must
+			// The external port cannot be a wildcard, but must
 			// be specified
 			external_port_cannot_be_wildcarded = 716,
 			// The port mapping entry specified conflicts with a
@@ -82,7 +93,7 @@ namespace libtorrent
 			// lease times on port mappings
 			only_permanent_leases_supported = 725,
 			// RemoteHost must be a wildcard and cannot be a
-			// specific IP addres or DNS name
+			// specific IP address or DNS name
 			remote_host_must_be_wildcard = 726,
 			// ExternalPort must be a wildcard and cannot be a
 			// specific port
@@ -96,81 +107,68 @@ namespace libtorrent
 	// the boost.system error category for UPnP errors
 	TORRENT_EXPORT boost::system::error_category& upnp_category();
 
-#ifndef TORRENT_NO_DEPRECATED
-	TORRENT_DEPRECATED TORRENT_EXPORT
-	boost::system::error_category& get_upnp_category();
+#if TORRENT_ABI_VERSION == 1
+	TORRENT_DEPRECATED
+	inline boost::system::error_category& get_upnp_category()
+	{ return upnp_category(); }
 #endif
 
-// int: port-mapping index
-// address: external address as queried from router
-// int: external port
-// int: protocol (UDP, TCP)
-// std::string: error message
-// an empty string as error means success
-typedef boost::function<void(int, address, int, int, error_code const&)> portmap_callback_t;
-typedef boost::function<void(char const*)> log_callback_t;
-
 struct parse_state
 {
-	parse_state(): in_service(false) {}
-	bool in_service;
-	std::list<std::string> tag_stack;
+	bool in_service = false;
+	std::vector<string_view> tag_stack;
 	std::string control_url;
 	std::string service_type;
 	std::string model;
 	std::string url_base;
-	bool top_tags(const char* str1, const char* str2)
+	bool top_tags(string_view str1, string_view str2)
 	{
-		std::list<std::string>::reverse_iterator i = tag_stack.rbegin();
+		auto i = tag_stack.rbegin();
 		if (i == tag_stack.rend()) return false;
-		if (!string_equal_no_case(i->c_str(), str2)) return false;
+		if (!string_equal_no_case(*i, str2)) return false;
 		++i;
 		if (i == tag_stack.rend()) return false;
-		if (!string_equal_no_case(i->c_str(), str1)) return false;
+		if (!string_equal_no_case(*i, str1)) return false;
 		return true;
 	}
 };
 
 struct error_code_parse_state
 {
-	error_code_parse_state(): in_error_code(false), exit(false), error_code(-1) {}
-	bool in_error_code;
-	bool exit;
-	int error_code;
+	bool in_error_code = false;
+	bool exit = false;
+	int error_code = -1;
 };
 
 struct ip_address_parse_state: error_code_parse_state
 {
-	ip_address_parse_state(): in_ip_address(false) {}
-	bool in_ip_address;
+	bool in_ip_address = false;
 	std::string ip_address;
 };
 
-TORRENT_EXTRA_EXPORT void find_control_url(int type, char const* string
-	, int str_len, parse_state& state);
+TORRENT_EXTRA_EXPORT void find_control_url(int type, string_view, parse_state& state);
 
-TORRENT_EXTRA_EXPORT void find_error_code(int type, char const* string
-	, int str_len, error_code_parse_state& state);
+TORRENT_EXTRA_EXPORT void find_error_code(int type, string_view string
+	, error_code_parse_state& state);
 
-TORRENT_EXTRA_EXPORT void find_ip_address(int type, char const* string
-	, int str_len, ip_address_parse_state& state);
+TORRENT_EXTRA_EXPORT void find_ip_address(int type, string_view string
+	, ip_address_parse_state& state);
 
 // TODO: support using the windows API for UPnP operations as well
-class TORRENT_EXTRA_EXPORT upnp : public boost::enable_shared_from_this<upnp>
+struct TORRENT_EXTRA_EXPORT upnp final
+	: std::enable_shared_from_this<upnp>
+	, single_threaded
 {
-public:
 	upnp(io_service& ios
-		, address const& listen_interface, std::string const& user_agent
-		, portmap_callback_t const& cb, log_callback_t const& lcb
-		, bool ignore_nonrouters);
+		, aux::session_settings const& settings
+		, aux::portmap_callback& cb
+		, address_v4 const& listen_address
+		, address_v4 const& netmask
+		, std::string listen_device);
 	~upnp();
 
-	void set_user_agent(std::string const& v) { m_user_agent = v; }
-
 	void start();
 
-	enum protocol_type { none = 0, udp = 1, tcp = 2 };
-
 	// Attempts to add a port mapping for the specified protocol. Valid protocols are
 	// ``upnp::tcp`` and ``upnp::udp`` for the UPnP class and ``natpmp::tcp`` and
 	// ``natpmp::udp`` for the NAT-PMP class.
@@ -188,48 +186,47 @@ public:
 	// portmap_alert_ respectively. If The mapping fails immediately, the return value
 	// is -1, which means failure. There will not be any error alert notification for
 	// mappings that fail with a -1 return value.
-	int add_mapping(protocol_type p, int external_port, tcp::endpoint local_ep);
+	port_mapping_t add_mapping(portmap_protocol p, int external_port, tcp::endpoint local_ep
+		, std::string const& device);
 
 	// This function removes a port mapping. ``mapping_index`` is the index that refers
 	// to the mapping you want to remove, which was returned from add_mapping().
-	void delete_mapping(int mapping_index);
+	void delete_mapping(port_mapping_t mapping_index);
 
-	bool get_mapping(int mapping_index, tcp::endpoint& local_ep, int& external_port, int& protocol) const;
+	bool get_mapping(port_mapping_t mapping_index, tcp::endpoint& local_ep, int& external_port
+		, portmap_protocol& protocol) const;
 
-	void discover_device();
 	void close();
 
-	// This is only available for UPnP routers. If the model is advertized by
+	// This is only available for UPnP routers. If the model is advertised by
 	// the router, it can be queried through this function.
 	std::string router_model()
 	{
-		mutex::scoped_lock l(m_mutex);
+		TORRENT_ASSERT(is_single_thread());
 		return m_model;
 	}
 
 private:
 
-	boost::shared_ptr<upnp> self() { return shared_from_this(); }
+	std::shared_ptr<upnp> self() { return shared_from_this(); }
+
+	void open_multicast_socket(aux::socket_package& s, error_code& ec);
+	void open_unicast_socket(aux::socket_package& s, error_code& ec);
 
 	void map_timer(error_code const& ec);
-	void try_map_upnp(mutex::scoped_lock& l, bool timer = false);
-	void discover_device_impl(mutex::scoped_lock& l);
-	static address_v4 upnp_multicast_address;
-	static udp::endpoint upnp_multicast_endpoint;
-
-	// there are routers that's don't support timed
-	// port maps, without returning error 725. It seems
-	// safer to always assume that we have to ask for
-	// permanent leases
-	enum { default_lease_time = 0 };
+	void try_map_upnp();
+	void discover_device_impl();
 
 	void resend_request(error_code const& e);
-	void on_reply(udp::endpoint const& from, char* buffer
-		, std::size_t bytes_transferred);
+	void on_reply(aux::socket_package& s, error_code const& ec, std::size_t len);
 
 	struct rootdevice;
-	void next(rootdevice& d, int i, mutex::scoped_lock& l);
-	void update_map(rootdevice& d, int i, mutex::scoped_lock& l);
+	void next(rootdevice& d, port_mapping_t i);
+	void update_map(rootdevice& d, port_mapping_t i);
+
+	int lease_duration(rootdevice const& d) const;
+
+	void connect(rootdevice& d);
 
 	void on_upnp_xml(error_code const& e
 		, libtorrent::http_parser const& p, rootdevice& d
@@ -239,77 +236,58 @@ private:
 		, http_connection& c);
 	void on_upnp_map_response(error_code const& e
 		, libtorrent::http_parser const& p, rootdevice& d
-		, int mapping, http_connection& c);
+		, port_mapping_t mapping, http_connection& c);
 	void on_upnp_unmap_response(error_code const& e
 		, libtorrent::http_parser const& p, rootdevice& d
-		, int mapping, http_connection& c);
+		, port_mapping_t mapping, http_connection& c);
 	void on_expire(error_code const& e);
 
-	void disable(error_code const& ec, mutex::scoped_lock& l);
-	void return_error(int mapping, int code, mutex::scoped_lock& l);
-	void log(char const* msg, mutex::scoped_lock& l);
+	void disable(error_code const& ec);
+	void return_error(port_mapping_t mapping, int code);
+#ifndef TORRENT_DISABLE_LOGGING
+	bool should_log() const;
+	void log(char const* msg, ...) const TORRENT_FORMAT(2,3);
+#endif
 
 	void get_ip_address(rootdevice& d);
-	void delete_port_mapping(rootdevice& d, int i);
-	void create_port_mapping(http_connection& c, rootdevice& d, int i);
+	void delete_port_mapping(rootdevice& d, port_mapping_t i);
+	void create_port_mapping(http_connection& c, rootdevice& d, port_mapping_t i);
 	void post(upnp::rootdevice const& d, char const* soap
-		, char const* soap_action, mutex::scoped_lock& l);
+		, char const* soap_action);
 
 	int num_mappings() const { return int(m_mappings.size()); }
 
 	struct global_mapping_t
 	{
-		global_mapping_t()
-			: protocol(none)
-			, external_port(0)
-		{}
-		int protocol;
-		int external_port;
+		portmap_protocol protocol = portmap_protocol::none;
+		int external_port = 0;
 		tcp::endpoint local_ep;
+		// may be set to a device name, if this mapping is for a network bound
+		// to a specific network device
+		std::string device;
 	};
 
-	struct mapping_t
+	struct mapping_t : aux::base_mapping
 	{
-		enum action_t { action_none, action_add, action_delete };
-		mapping_t()
-			: action(action_none)
-			, external_port(0)
-			, protocol(none)
-			, failcount(0)
-		{}
-
-		// the time the port mapping will expire
-		time_point expires;
-
-		// the local port for this mapping. The port is set
+		// the local port for this mapping. If this is set
 		// to 0, the mapping is not in use
 		tcp::endpoint local_ep;
 
-		int action;
-
-		// the external (on the NAT router) port
-		// for the mapping. This is the port we
-		// should announce to others
-		int external_port;
-
-		// 2 = udp, 1 = tcp
-		int protocol;
+		// may be set to a network device name to bind to
+		std::string device;
 
 		// the number of times this mapping has failed
-		int failcount;
+		int failcount = 0;
 	};
 
 	struct rootdevice
 	{
 		rootdevice();
-
-#if TORRENT_USE_ASSERTS
 		~rootdevice();
-#if __cplusplus >= 201103L
 		rootdevice(rootdevice const&);
 		rootdevice& operator=(rootdevice const&);
-#endif
-#endif
+		rootdevice(rootdevice&&);
+		rootdevice& operator=(rootdevice&&);
 
 		// the interface url, through which the list of
 		// supported interfaces are fetched
@@ -320,36 +298,30 @@ private:
 		// either the WANIP namespace or the WANPPP namespace
 		std::string service_namespace;
 
-		std::vector<mapping_t> mapping;
+		aux::vector<mapping_t, port_mapping_t> mapping;
 
 		// this is the hostname, port and path
 		// component of the url or the control_url
 		// if it has been found
 		std::string hostname;
-		int port;
+		int port = 0;
 		std::string path;
 		address external_ip;
 
-		int lease_duration;
+		// set to false if the router doesn't support lease durations
+		bool use_lease_duration = true;
+
 		// true if the device supports specifying a
 		// specific external port, false if it doesn't
-		bool supports_specific_external;
+		bool supports_specific_external = true;
 
-		bool disabled;
+		bool disabled = false;
 
-		// this is true if the IP of this device is not
-		// one of our default routes. i.e. it may be someone
-		// else's router, we just happen to have multicast
-		// enabled across networks
-		// this is only relevant if ignore_non_routers is set.
-		bool non_router;
-
-		mutable boost::shared_ptr<http_connection> upnp_connection;
+		mutable std::shared_ptr<http_connection> upnp_connection;
 
 #if TORRENT_USE_ASSERTS
-		int magic;
+		int magic = 1337;
 #endif
-		void close() const;
 
 		bool operator<(rootdevice const& rhs) const
 		{ return url < rhs.url; }
@@ -357,22 +329,21 @@ private:
 
 	struct upnp_state_t
 	{
-		std::vector<global_mapping_t> mappings;
+		aux::vector<global_mapping_t, port_mapping_t> mappings;
 		std::set<rootdevice> devices;
 	};
 
-	std::vector<global_mapping_t> m_mappings;
+	aux::vector<global_mapping_t, port_mapping_t> m_mappings;
 
-	std::string m_user_agent;
+	aux::session_settings const& m_settings;
 
 	// the set of devices we've found
 	std::set<rootdevice> m_devices;
 
-	portmap_callback_t m_callback;
-	log_callback_t m_log_callback;
+	aux::portmap_callback& m_callback;
 
 	// current retry count
-	int m_retry_count;
+	int m_retry_count = 0;
 
 	io_service& m_io_service;
 
@@ -380,7 +351,8 @@ private:
 
 	// the udp socket used to send and receive
 	// multicast messages on the network
-	broadcast_socket m_socket;
+	aux::socket_package m_multicast;
+	aux::socket_package m_unicast;
 
 	// used to resend udp packets in case
 	// they time out
@@ -390,23 +362,26 @@ private:
 	deadline_timer m_refresh_timer;
 
 	// this timer fires one second after the last UPnP response. This is the
-	// point where we assume we have received most or all SSDP reponses. If we
+	// point where we assume we have received most or all SSDP responses. If we
 	// are ignoring non-routers and at this point we still haven't received a
 	// response from a router UPnP device, we override the ignoring behavior and
 	// map them anyway.
 	deadline_timer m_map_timer;
 
-	bool m_disabled;
-	bool m_closing;
-	bool m_ignore_non_routers;
-
-	mutex m_mutex;
+	bool m_disabled = false;
+	bool m_closing = false;
 
 	std::string m_model;
 
-	// cache of interfaces
-	mutable std::vector<ip_interface> m_interfaces;
-	mutable time_point m_last_if_update;
+	// the network this UPnP mapper is associated with. Don't talk to any other
+	// network
+	address_v4 m_listen_address;
+	address_v4 m_netmask;
+	std::string m_device;
+
+#ifdef TORRENT_USE_OPENSSL
+	ssl::context m_ssl_ctx;
+#endif
 };
 
 }
@@ -419,4 +394,3 @@ namespace boost { namespace system {
 } }
 
 #endif
-
