@@ -49,6 +49,9 @@ namespace libtorrent {
 
 namespace {
 
+	static_assert(int('0') == 48, "this code relies on ASCII compatible source encoding");
+	static_assert(int('9') == 57, "this code relies on ASCII compatible source encoding");
+
 	bool numeric(char c) { return c >= '0' && c <= '9'; }
 
 	// finds the end of an integer and verifies that it looks valid this does
@@ -148,12 +151,16 @@ namespace detail {
 	// non-digit in the range, a parse error is reported in 'ec'. If the value
 	// cannot be represented by the variable 'val' and overflow error is reported
 	// by 'ec'.
+	// WARNING: the val is an out-parameter that is expected to be initialized
+	// to 0 or a *positive* number.
 	char const* parse_int(char const* start, char const* end, char delimiter
 		, std::int64_t& val, bdecode_errors::error_code_enum& ec)
 	{
+		TORRENT_ASSERT(val >= 0);
 		while (start < end && *start != delimiter)
 		{
-			if (!numeric(*start))
+			char const c = *start;
+			if (!numeric(c))
 			{
 				ec = bdecode_errors::expected_digit;
 				return start;
@@ -164,7 +171,7 @@ namespace detail {
 				return start;
 			}
 			val *= 10;
-			int digit = *start - '0';
+			int const digit = c - '0';
 			if (val > std::numeric_limits<std::int64_t>::max() - digit)
 			{
 				ec = bdecode_errors::overflow;
@@ -326,6 +333,7 @@ namespace detail {
 				}
 				break;
 			case bdecode_token::string:
+			case bdecode_token::long_string:
 				if (m_buffer[tokens[token].offset] == '0'
 					&& m_buffer[tokens[token].offset + 1] != ':')
 				{
@@ -392,6 +400,8 @@ namespace detail {
 	bdecode_node::type_t bdecode_node::type() const noexcept
 	{
 		if (m_token_idx == -1) return none_t;
+		if (m_root_tokens[m_token_idx].type == bdecode_token::long_string)
+			return bdecode_node::string_t;
 		return static_cast<bdecode_node::type_t>(m_root_tokens[m_token_idx].type);
 	}
 
@@ -509,7 +519,8 @@ namespace detail {
 
 		while (item < i)
 		{
-			TORRENT_ASSERT(tokens[token].type == bdecode_token::string);
+			TORRENT_ASSERT(tokens[token].type == bdecode_token::string
+				|| tokens[token].type == bdecode_token::long_string);
 
 			// skip the key
 			token += tokens[token].next_item;
@@ -589,7 +600,8 @@ namespace detail {
 		while (tokens[token].type != bdecode_token::end)
 		{
 			bdecode_token const& t = tokens[token];
-			TORRENT_ASSERT(t.type == bdecode_token::string);
+			TORRENT_ASSERT(t.type == bdecode_token::string
+				|| t.type == bdecode_token::long_string);
 			int const size = token_source_span(t) - t.start_offset();
 			if (int(key.size()) == size
 				&& std::equal(key.data(), key.data() + size, m_buffer
@@ -687,7 +699,8 @@ namespace detail {
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
 		std::size_t const size = aux::numeric_cast<std::size_t>(token_source_span(t) - t.start_offset());
-		TORRENT_ASSERT(t.type == bdecode_token::string);
+		TORRENT_ASSERT(t.type == bdecode_token::string
+			|| t.type == bdecode_token::long_string);
 
 		return string_view(m_buffer + t.offset + t.start_offset(), size);
 	}
@@ -696,7 +709,8 @@ namespace detail {
 	{
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
-		TORRENT_ASSERT(t.type == bdecode_token::string);
+		TORRENT_ASSERT(t.type == bdecode_token::string
+			|| t.type == bdecode_token::long_string);
 		return m_buffer + t.offset + t.start_offset();
 	}
 
@@ -704,7 +718,8 @@ namespace detail {
 	{
 		TORRENT_ASSERT(type() == string_t);
 		bdecode_token const& t = m_root_tokens[m_token_idx];
-		TORRENT_ASSERT(t.type == bdecode_token::string);
+		TORRENT_ASSERT(t.type == bdecode_token::string
+			|| t.type == bdecode_token::long_string);
 		return token_source_span(t) - t.start_offset();
 	}
 
@@ -926,13 +941,16 @@ namespace detail {
 					// the check above ensures that the buffer is long enough to hold
 					// the string's length which guarantees that start <= end
 
+					int const header = int(start - str_start);
+					TORRENT_ASSERT(header >= 0);
+
 					// the bdecode_token only has 8 bits to keep the header size
 					// in. If it overflows, fail!
-					if (start - str_start - 2 > detail::bdecode_token::max_header)
+					if (header > detail::bdecode_token::long_string_max_header)
 						TORRENT_FAIL_BDECODE(bdecode_errors::limit_exceeded);
 
 					ret.m_tokens.push_back({str_start - orig_start
-						, 1, bdecode_token::string, std::uint8_t(start - str_start)});
+						, 1, bdecode_token::string, std::uint32_t(header)});
 					start += len;
 					break;
 				}
