@@ -43,6 +43,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #if TORRENT_ABI_VERSION == 1
 #include "libtorrent/read_resume_data.hpp"
 #include "libtorrent/lazy_entry.hpp"
+#include "libtorrent/magnet_uri.hpp"
 #endif
 
 using libtorrent::aux::session_impl;
@@ -71,6 +72,7 @@ namespace libtorrent {
 #if TORRENT_ABI_VERSION == 1
 	constexpr session_flags_t session_handle::start_default_features;
 #endif
+	constexpr session_flags_t session_handle::paused;
 
 	constexpr remove_flags_t session_handle::delete_files;
 	constexpr remove_flags_t session_handle::delete_partfile;
@@ -356,6 +358,14 @@ namespace {
 			if (atp.file_priorities.empty())
 				atp.file_priorities = resume_data.file_priorities;
 		}
+
+		if (atp.info_hash.is_all_zeros()
+			&& string_begins_no_case("magnet:", atp.url.c_str()))
+		{
+			error_code err;
+			parse_magnet_uri(atp.url, atp, err);
+			if (!err) atp.url.clear();
+		}
 	}
 
 } // anonymous namespace
@@ -367,6 +377,11 @@ namespace {
 	{
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
+		// the internal torrent object keeps and mutates state in the
+		// torrent_info object. We can't let that leak back to the client
+		if (params.ti)
+			params.ti = std::make_shared<torrent_info>(*params.ti);
+
 #if TORRENT_ABI_VERSION == 1
 		handle_backwards_compatible_resume_data(params);
 #endif
@@ -387,6 +402,11 @@ namespace {
 	{
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
+		// the internal torrent object keeps and mutates state in the
+		// torrent_info object. We can't let that leak back to the client
+		if (params.ti)
+			params.ti = std::make_shared<torrent_info>(*params.ti);
+
 		ec.clear();
 #if TORRENT_ABI_VERSION == 1
 		handle_backwards_compatible_resume_data(params);
@@ -409,6 +429,11 @@ namespace {
 	{
 		TORRENT_ASSERT_PRECOND(!params.save_path.empty());
 
+		// the internal torrent object keeps and mutates state in the
+		// torrent_info object. We can't let that leak back to the client
+		if (params.ti)
+			params.ti = std::make_shared<torrent_info>(*params.ti);
+
 		// we cannot capture a unique_ptr into a lambda in c++11, so we use a raw
 		// pointer for now. async_call uses a lambda expression to post the call
 		// to the main thread
@@ -433,7 +458,7 @@ namespace {
 		, std::string const& save_path
 		, entry const& resume_data
 		, storage_mode_t storage_mode
-		, bool paused
+		, bool add_paused
 		, storage_constructor_type sc)
 	{
 		add_torrent_params p(std::move(sc));
@@ -444,7 +469,7 @@ namespace {
 			bencode(std::back_inserter(p.resume_data), resume_data);
 		}
 		p.storage_mode = storage_mode;
-		if (paused) p.flags |= add_torrent_params::flag_paused;
+		if (add_paused) p.flags |= add_torrent_params::flag_paused;
 		else p.flags &= ~add_torrent_params::flag_paused;
 		return add_torrent(p);
 	}
@@ -456,7 +481,7 @@ namespace {
 		, std::string const& save_path
 		, entry const& resume_data
 		, storage_mode_t storage_mode
-		, bool paused
+		, bool add_paused
 		, storage_constructor_type sc
 		, void* userdata)
 	{
@@ -468,7 +493,7 @@ namespace {
 		p.save_path = save_path;
 		p.storage_mode = storage_mode;
 
-		if (paused) p.flags |= add_torrent_params::flag_paused;
+		if (add_paused) p.flags |= add_torrent_params::flag_paused;
 		else p.flags &= ~add_torrent_params::flag_paused;
 
 		p.userdata = userdata;
