@@ -724,7 +724,7 @@ namespace libtorrent {
 	{
 		add_torrent_params params;
 		auto retr = std::ref(params);
-		sync_call(&torrent::write_resume_data, retr);
+		sync_call(&torrent::write_resume_data, resume_data_flags_t{}, retr);
 		return libtorrent::write_resume_data(params);
 	}
 
@@ -863,6 +863,15 @@ namespace libtorrent {
 		return std::size_t(*reinterpret_cast<void* const*>(&th.m_torrent));
 	}
 
+	void* torrent_handle::userdata() const
+	{
+		std::shared_ptr<torrent> t = m_torrent.lock();
+		return t ? t->get_userdata() : nullptr;
+	}
+
+	bool torrent_handle::in_session() const
+	{ return !sync_call_ret<bool>(false, &torrent::is_aborted); }
+
 	static_assert(std::is_nothrow_move_constructible<torrent_handle>::value
 		, "should be nothrow move constructible");
 	static_assert(std::is_nothrow_move_assignable<torrent_handle>::value
