@@ -73,6 +73,8 @@ TORRENT_TEST(error_code)
 	TEST_CHECK(error_code(errors::no_i2p_router).message() == "no i2p router is set up");
 	TEST_CHECK(error_code(errors::http_parse_error).message() == "Invalid HTTP header");
 	TEST_CHECK(error_code(errors::error_code_max).message() == "Unknown error");
+	TEST_CHECK(error_code(errors::ssrf_mitigation).message() == "blocked by SSRF mitigation");
+	TEST_CHECK(error_code(errors::blocked_by_idna).message() == "blocked by IDNA ban");
 
 	TEST_CHECK(error_code(errors::unauthorized, http_category()).message()
 		== "401 Unauthorized");
